using System;
using System.Web;
using System.Resources;
using System.Reflection;
using System.Globalization;
using System.Configuration;

namespace GlobalizationModule
{
	/// <summary>
	/// GlobalizationMod is an IHttpModule implementation. It does the
	/// leg work of processing the Localize attributes on a web page
	/// and putting the localized values into the controls.
	/// </summary>
	public class GlobalizationMod : IHttpModule
	{
		public GlobalizationMod()
		{}

        HttpApplication app_;
        System.Web.UI.Page page_;

        /// <summary>
        /// Init prepares to handle the PreRequestHandlerExecute event
        /// of the HttpApplication.
        /// </summary>
        public void Init(HttpApplication app)
        {
            app_ = app;
            app_.PreRequestHandlerExecute += new EventHandler(this.OnPreRequest);
        }

        public void Dispose()
        {
            app_.PreRequestHandlerExecute -= new EventHandler(this.OnPreRequest);
            page_.PreRender -= new EventHandler(this.OnPreRender);
        }

        /// <summary>
        /// PreRequest prepares to handle the PreRender event of the page.
        /// </summary>
        /// <param name="sender">HttpApplication</param>
        /// <param name="eventArgs"></param>
        public void OnPreRequest(object sender, EventArgs eventArgs)
        {
            // Get the IHttpHandler from the HttpApplication
            HttpContext ctx = ((HttpApplication)sender).Context;
            IHttpHandler handler = ctx.Handler;

            // Handle the PreRender event of the page
            if (handler is System.Web.UI.Page)
            {
                page_ = (System.Web.UI.Page)handler;
                page_.PreRender += new EventHandler(this.OnPreRender);
            }
        }

        // Remember user's page class
        private System.Web.UI.Page targetPage_;
        // Remember root attribute
        private LocalizeAttribute rootAttribute_;

        /// <summary>
        /// OnPreRender performs the localization logic.
        /// </summary>
        /// <param name="sender">Web page instance</param>
        /// <param name="eventArgs"></param>
        public void OnPreRender(object sender, EventArgs eventArgs)
        {
            // If current culture is the same as the web site's
            // default language, then we do nothing
            CultureInfo currentCulture = System.Threading.Thread.CurrentThread.CurrentUICulture;
            if (ConfigurationSettings.AppSettings["DefaultLanguage"] == currentCulture.Name)
                return;

            System.Web.UI.Page page = (System.Web.UI.Page)sender;

            
            // Localize the page if it has the Localize attribute
            object[] typeAttrs = page.GetType().GetCustomAttributes(typeof(LocalizeAttribute), true);
            if (typeAttrs != null && typeAttrs.Length > 0)
            {
                LocalizeObject(null, sender, (LocalizeAttribute)typeAttrs[0]);
            }            
        }

        /// <summary>
        /// Localize target based on attribute.
        /// </summary>
        /// <param name="fieldInfo">Reflection.FieldInfo with information about 'target'</param>
        /// <param name="target">Object to localize</param>
        /// <param name="attr">Localization attribute</param>
        protected void LocalizeObject(FieldInfo fieldInfo, object target, LocalizeAttribute attr)
        {
            if (attr.Mode == LocalizeAttribute.LocalizeMode.Fields)
            {
                //
                // Localize child objects of target
                //
                FieldInfo[] fields = null;
                Type targetType = null;

                if (target is System.Web.UI.Page)
                {   // Remember the web page class
                    targetPage_ = (System.Web.UI.Page)target;
                    // Remember root attribute
                    rootAttribute_ = attr;
                    targetType = target.GetType().BaseType;
                }
                else
                {
                    targetType = target.GetType();
                }

                //
                // Localize fields that have the Localize attribute
                //
                fields = targetType.GetFields(BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
                foreach (FieldInfo f in fields)
                {
                    // Get the child instance
                    object child = f.GetValue(target);
                    if (child != null)
                    {                     
                        // Localize this object if it has the Localize attribute
                        object[] typeAttrs = f.GetCustomAttributes(typeof(LocalizeAttribute), false);
                        if (typeAttrs != null && typeAttrs.Length > 0)
                        {
                            LocalizeObject(f, child, (LocalizeAttribute)typeAttrs[0]);
                        }
                    }
                }
            }
            else // attr.Mode != LocalizeAttribute.LocalizeMode.Fields
            {
                // If this attribute has no resource name specified,
                // use the root attribute's resource name as base resource name.
                if (attr.ResourceBaseName == null)
                    attr.ResourceBaseName = rootAttribute_.ResourceBaseName;

                // If this attribute has no resource name specified,
                // use the target object's name as resource name.
                if (attr.ResourceName == null)
                    attr.ResourceName = fieldInfo.Name;

                // The actual localization of the target is performed
                // by the attribute. Seems odd? Well, this way the developer
                // can implement his/her own localization logic by deriving a
                // class from LocalizeAttribute and overriding the Localize
                // method.
                attr.LocalizeObject(target, targetPage_);
            }
        }
	}
}
